/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"

#include <chrono>
#include <string>

#include "Containers/UnrealString.h"
#include "Misc/App.h"
#include "Misc/ConfigCacheIni.h"
#include "Engine/Engine.h"
#include "Engine/GameViewportClient.h"
#include "Widgets/SWindow.h"
#include "Runtime/Launch/Resources/Version.h"
#include "Windows/WindowsPlatformProcess.h"
#include "Windows/WindowsPlatformMisc.h"


#include "MixCast/Public/IMixCast.h"

const FString SDK_VERSION = FString::Printf(TEXT("%d.%d.%d"), MIXCAST_SDK_MAJOR_VERSION, MIXCAST_SDK_MINOR_VERSION, MIXCAST_SDK_PATCH_VERSION);
const FString UNREAL_VERSION = FString::Printf(TEXT("%d.%d.%d"), ENGINE_MAJOR_VERSION, ENGINE_MINOR_VERSION, ENGINE_PATCH_VERSION);

class MIXCAST_API MiscUtil
{
public:
	static FString GetCompanyName()
	{
		FString CompanyName;
		GConfig->GetString(TEXT("/Script/EngineSettings.GeneralProjectSettings"), TEXT("CompanyName"), CompanyName, GGameIni);

		if (CompanyName.Len() == 0)
		{
			CompanyName = FString(TEXT("DefaultCompany"));
		}

		return CompanyName;
	}

	static FString GetDeviceId()
	{
		PRAGMA_DISABLE_DEPRECATION_WARNINGS
		FString DeviceId = FPlatformMisc::GetMacAddressString();
		PRAGMA_ENABLE_DEPRECATION_WARNINGS

		// The device ID uses the MAC address, which is not guaranteed to be supported on all platforms.
		if (DeviceId.Len() == 0)
		{
			DeviceId = FString(TEXT("NoDeviceId"));
		}

		return DeviceId;
	}

	static FString GetExecutablePath()
	{
		if (FApp::IsGame())
		{
#if ENGINE_MAJOR_VERSION == 4 && ENGINE_MINOR_VERSION >= 22
			return FString(FWindowsPlatformProcess::ExecutablePath());
#else
			return FWindowsPlatformProcess::GetApplicationName(FWindowsPlatformProcess::GetCurrentProcessId());
#endif
		}
		else
		{
			return FPaths::Combine(FPlatformProcess::BaseDir(), FPlatformProcess::ExecutableName(false));
		}
	}

	static uint64 GetMainWindowHandle()
	{
		if (!GEngine || !GEngine->GameViewport || !GEngine->GameViewport->GetWindow().IsValid() || !GEngine->GameViewport->GetWindow()->GetNativeWindow().IsValid())
			return 0;
		const void* hwnd = GEngine->GameViewport->GetWindow()->GetNativeWindow()->GetOSWindowHandle();
		return (uint64)hwnd;
	}

	static FString GetProjectName()
	{
#if ENGINE_MAJOR_VERSION == 4 && ENGINE_MINOR_VERSION >= 18
		return FString(FApp::GetProjectName());
#else
		return FString(FApp::GetGameName());
#endif
	}

	static FString GetProjectID()
	{
		FString ProjectID;
		GConfig->GetString(TEXT("/Script/EngineSettings.GeneralProjectSettings"), TEXT("ProjectID"), ProjectID, GGameIni);

		if (ProjectID.Len() == 0)
		{
			ProjectID = FString(TEXT("---"));
		}

		return ProjectID;
	}

	static std::string ToStdString(const FString& String)
	{
		return std::string(TCHAR_TO_UTF8(*String));
	}

	static bool StdAndFStrAreEqual(const std::string& stdStr, const FString& fStr)
	{
		if (stdStr.length() != fStr.Len())
			return false;

		int fStrCharIndex = 0;
		for (char const &c : stdStr)
		{
			if (c != fStr[fStrCharIndex++])
				return false;
		}
		return true;
	}

	static bool IsVersionBLaterThanVersionA(FString versionA, FString versionB)
	{
		if (versionA.Compare(versionB) == 0)
			return false;

		TArray<FString> aNums, bNums;
		versionA.ParseIntoArray(aNums, TEXT("."));
		versionB.ParseIntoArray(bNums, TEXT("."));

		for (int i = 0; i < aNums.Num() && i < bNums.Num(); i++)
		{
			int aNum = FCString::Atoi(*aNums[i]);
			int bNum = FCString::Atoi(*bNums[i]);

			if (bNum > aNum)
				return true;
			else if (bNum < aNum)
				return false;
		}

		if (bNums.Num() > aNums.Num())
			return true;

		return false;
	}

	static unsigned long long GetMixCastTimestamp()
	{
		return std::chrono::system_clock::now().time_since_epoch().count();
	}
};
